@@ -1,11 +1,11 @@
 (* approx: proxy server for Debian archive files
-   Copyright (C) 2014  Eric C. Cooper <ecc@cmu.edu>
+   Copyright (C) 2016  Eric C. Cooper <ecc@cmu.edu>
    Released under the GNU General Public License *)
 
 open Config_file
 open Util
 
-let version = "5.5"
+let version = "5.7"
 
 let default_config = "/etc/approx/approx.conf"
 
@@ -31,21 +31,34 @@ let () =
 
 let params = []
 
-let cache_dir = get "$cache" ~default: "/var/cache/approx"
+let cache_dir =
+  let dir = remove_trailing '/' (get "$cache" ~default: "/var/cache/approx") in
+  let n = String.length dir in
+  if n > 0 && dir.[0] = '/' then dir
+  else invalid_arg "$cache"
+
 let params = ("$cache", cache_dir) :: params
 
 let split_cache_path path =
   let err () = invalid_string_arg "split_cache_path" path in
-  if is_prefix cache_dir path then
-    let i = String.length cache_dir + 1 in
-    let j = try String.index_from path i '/' with Not_found -> err () in
-    substring path ~from: i ~until: j, substring path ~from: (j + 1)
+  let dir = cache_dir ^ "/" in
+  if is_prefix dir path then
+    let i = String.length dir in
+    let rest = remove_leading '/' (substring path ~from: i) in
+    let j = try String.index rest '/' with Not_found -> err () in
+    match (substring rest ~until: j,
+           remove_leading '/' (substring rest ~from: (j + 1))) with
+    | ("", _) | (_, "") -> err ()
+    | pair -> pair
   else
     err ()
 
 let shorten path =
-  if is_prefix cache_dir path then
-    substring path ~from: (String.length cache_dir + 1)
+  let dir = cache_dir ^ "/" in
+  if is_prefix dir path then
+    match remove_leading '/' (substring path ~from: (String.length dir)) with
+    | "" -> path
+    | str -> str
   else
     path
 
@@ -76,6 +89,9 @@ let params = ("$offline", string_of_bool offline) :: params
 let max_wait = get_int "$max_wait" ~default: 10 (* seconds *)
 let params = ("$max_wait", string_of_int max_wait) :: params
 
+let curl_path = get "$curl_path" ~default: "/usr/bin/curl"
+let params = ("$curl_path", curl_path) :: params
+
 let debug = get_bool "$debug" ~default: false
 let params = ("$debug", string_of_bool debug) :: params
 
